package com.persagy.apm.diagnose.executor;

import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicLong;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName ThreadPoolExecutorExtend.java
 * @Description 自定义线程池,实现监控线程池方法
 * @createTime 2021年09月20日 21:23:00
 */
public class ThreadPoolExecutorExtend extends ThreadPoolExecutor {
    private final ThreadLocal startTime = new ThreadLocal();
    private final AtomicLong numTasks = new AtomicLong();
    private final AtomicLong totalTime = new AtomicLong();

    public ThreadPoolExecutorExtend(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue);
    }

    public ThreadPoolExecutorExtend(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, ThreadFactory threadFactory) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory);
    }

    public ThreadPoolExecutorExtend(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, RejectedExecutionHandler handler) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, handler);
    }

    public ThreadPoolExecutorExtend(int corePoolSize, int maximumPoolSize, long keepAliveTime, TimeUnit unit, BlockingQueue<Runnable> workQueue, ThreadFactory threadFactory, RejectedExecutionHandler handler) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory, handler);
    }

    @Override
    protected void afterExecute(Runnable r, Throwable t) {
        try{
            long endTime = System.currentTimeMillis();
            long useTime = endTime - (long)startTime.get();
            numTasks.incrementAndGet();
            totalTime.addAndGet(useTime);
            System.out.println("afterExecute " + r);
        }finally{
            super.afterExecute(r, t);
        }
    }

    @Override
    protected void beforeExecute(Thread t, Runnable r) {
        super.beforeExecute(t, r);
        System.out.println("beforeExecute " + r);
        startTime.set(System.currentTimeMillis());
    }

    @Override
    protected void terminated() {
        try{
            System.out.println("terminated avg time " + totalTime.get()  + " " + numTasks.get());
        }finally{
            super.terminated();
        }
    }
}
